% ------------------------------------------------------------------- %
% ------------------------------------------------------------------- %
% ------------------------------------------------------------------- %
% This function constructs a dataset from randomly selected salient   %
% subregions of randomly selected training images, learns a deep      %
% belief network of RBMs, and saves the corresponding weights         %
%                                                                     %
%                                                                     %
% selectedTrainingImageFilenames : Filenames of randomly selected     %
%                     training images                                 %
% minCircleRadius   : The size of the smallest circular object        %
%                     (tissue component) to be located                %
% winSize           : The size of a salient subregion                 %
%                                                                     %
%                                                                     %
% weightMatFile     : The output file that will be created to store   %
%                     the weights of RBMs. The bias vector names will %
%                     be B1, B2, ... and the weight matrix names will %
%                     be W1, W2, ...                                  %
%                     if you want to change the number of RBMs, or if %
%                     you want to change the number of hidden units,  %
%                     you should modify the code given in Lines 34-53 %
%                                                                     %
%                                                                     %
% ------------------------------------------------------------------- %
% ------------------------------------------------------------------- %
% ------------------------------------------------------------------- %
function learnDBN(selectedTrainingImageFilenames, weightMatFilename, minCircleRadius, winSize)

numberOfRandomlySelectedSubregions = 100;

trBatches = prepareBatchesForTraining(selectedTrainingImageFilenames, minCircleRadius, winSize, numberOfRandomlySelectedSubregions);

maxEpoch = 100;
hno1 = 2000;    % Number of hidden units in the 1st RBM
hno2 = 1000;    % Number of hidden units in the 2nd RBM
hno3 = 500;     % Number of hidden units in the 3rd RBM
hno4 = 100;     % Number of hidden units in the 4th RBM
  
[W1, A1, B1, batchposP1] = rbmFunc(maxEpoch, hno1, trBatches);
[W2, A2, B2, batchposP2] = rbmFunc(maxEpoch, hno2, batchposP1);
[W3, A3, B3, batchposP3] = rbmFunc(maxEpoch, hno3, batchposP2);
[W4, A4, B4, batchposP4] = rbmFunc(maxEpoch, hno4, batchposP3);
saveWeights(weightMatFilename, W1, A1, B1, W2, A2, B2, W3, A3, B3, W4, A4, B4);

end
%---------------------------------------------------------------------%
%---------------------------------------------------------------------%
%---------------------------------------------------------------------%
function saveWeights(weightMatFilename, W1, A1, B1, W2, A2, B2, W3, A3, B3, W4, A4, B4)

eval(['save ' weightMatFilename])

end
%---------------------------------------------------------------------%
%---------------------------------------------------------------------%
%---------------------------------------------------------------------%
function trBatches = prepareBatchesForTraining(selectedTrainingImageFilenames, minCircleRadius, winSize, batchNo)

trSize = size(selectedTrainingImageFilenames, 2);
firstIm = imread(selectedTrainingImageFilenames{1});
[dimx, dimy, ~] = size(firstIm);
halfSize    = (winSize - 1) / 2;
subLength   = (halfSize * 2 + 1) * (halfSize * 2 + 1);

trBatches     = zeros(trSize, winSize * winSize, batchNo);
for cnt = 1 : trSize
    
    im = imread(selectedTrainingImageFilenames{cnt});
    npix = findNucleusPixels(im);
    % im_circles is the file that temporarily stores the circles of the
    % current image. If you want to  store the circle information, use a
    % separate name for each image file. This will also fasten your program
    % if you want to run the algorithm with different parameters
    locateTissueCircles(im, 'H&E', 1, minCircleRadius, minCircleRadius, -1, 'im_circles', 0);
    orgCircles = load('im_circles');
        
    % Randomly select the circles for constructing the training set of
    % salient subregions
    cno = size(orgCircles, 1);
    cperm = randperm(cno);
    j = 1;
    selJ = 1;
    while (selJ < batchNo)
        cx = orgCircles(cperm(j), 1);
        cy = orgCircles(cperm(j), 2);
        
        if (cx - halfSize >= 1) && (cx + halfSize <= dimx) && (cy - halfSize >= 1) && (cy + halfSize <= dimy)
            trBatches(cnt,:,selJ) = reshape(npix(cx-halfSize : cx+halfSize, cy-halfSize : cy+halfSize),[subLength 1]);
            selJ = selJ + 1;
        end
        j = j + 1;
    end
end

end
%---------------------------------------------------------------------%
%---------------------------------------------------------------------%
%---------------------------------------------------------------------%


