% im                : rgb image matrix, NxMx3
% deconvType        : H&E / H&E2 / H&E3 for different coefficients
% thrMethod         : 0 for otsu, 1 for avg
%
% thrNc             : min radius for nucleus circles
% thrLm             : min radius for lumen circles
% thrSt             : min radius for stroma circles 
%                     (-1 if you want to ignore stroma circles)
%
% to save circles in a file, provide these parameters
%
% objFname          : filename to store the object information
%                     centroidX, centroidY, radius, type
%                     type --> 0 (nucleus), 1 (lumen), 2 (stroma, if any)
% objHeaderFlag     : 0 --> objFname will not have a header line
%                     1 --> objFname will have a header line 
%                     header line includes no of objects
%
% to run this program locateCircles.mexmaci64 (or circleFit.* in another OS)
%
function [all_circles, all_cmap, qpix] = locateTissueCircles(im, deconvType, thrMethod, thrNc, thrLm, thrSt, objFname, objHeaderFlag)

thrMethodHema = thrMethod;  % 0 for otsu, 1 for avg 
thrMethodEos  = thrMethod;  % 

%-----------------------------------------------%
%            quantize pixels into 3             %
%-----------------------------------------------%
newIm = stainDeconvolution(im,deconvType);
qpix  = thresholdHema(newIm, thrMethodHema);
if (thrSt ~= -1)
    qpix = thresholdEos(newIm, qpix, thrMethodEos);
end

%-----------------------------------------------%
%            find nucleus circles               %
%-----------------------------------------------%
cmap = (qpix == 0);
[nc_circles, nc_cmap] = locateCircles(double(cmap),thrNc);
nc_circles(:,4) = 0;

all_circles = nc_circles;
all_cmap = nc_cmap;
all_cmap(nc_cmap >= 0) = 0;
%-----------------------------------------------%
%            find lumen circles                 %
%-----------------------------------------------%
if (thrLm ~= -1)
    cmap = (qpix == 1);
    [lm_circles, lm_cmap] = locateCircles(double(cmap),thrLm);
    lm_circles(:,4) = 1;

    cno = size(all_circles,1);
    lno = size(lm_circles,1);
    all_circles(cno+1 : cno+lno, :) = lm_circles;
    all_cmap(lm_cmap >= 0) = 1;
    %-----------------------------------------------%
    %            find stroma circles                %
    %-----------------------------------------------%
    if (thrSt ~= -1)
        cmap = (qpix == 2);
        [st_circles, st_cmap] = locateCircles(double(cmap),thrSt);
        st_circles(:,4) = 2;
    
        cno = size(all_circles,1);
        sno = size(st_circles,1);
        all_circles(cno+1 : cno+sno, :) = st_circles;
        all_cmap(st_cmap >= 0) = 2;
    end
end

%-----------------------------------------------%
%            save circles in a file             %
%-----------------------------------------------%
if (nargin >= 7)
    saveCircles(all_circles, objFname, objHeaderFlag);
    
end

end
% --------------------------------------------------------------- %
% --------------------------------------------------------------- %
% --- For obtaining nucleus - lumen map on hematoxylin stain ---- %
% --------------------------------------------------------------- %
% --------------------------------------------------------------- %
function nc_lm = thresholdHema(newIm, thrMethod)

hema = newIm(:,:,1);
if (thrMethod == 0)
    thr_h = graythresh(hema);
    nc_lm = uint8(im2bw(hema,thr_h));
else
    thr_h = mean(hema(:));
    nc_lm = uint8(hema > thr_h);
end
   
end
% --------------------------------------------------------------- %
% --------------------------------------------------------------- %
% ------- For obtaining stroma - lumen map on eosin stain ------- %
% --------------------------------------------------------------- %
% --------------------------------------------------------------- %
function nc_lm_st = thresholdEos(newIm, nc_lm, thrMethod)

eos = newIm(:,:,2);
if (thrMethod == 0)
    thr_e = graythresh(eos);
    st = uint8(im2bw(eos,thr_e));
else
    thr_e = mean(eos(:));
    st = uint8(eos > thr_e);
end

nc_lm_st = nc_lm;
nc_lm_st(st == 0 & nc_lm == 1) = 2;

end
% --------------------------------------------------------------- %
% --------------------------------------------------------------- %
% ------------------- Save circles in a file -------------------- %
% --------------------------------------------------------------- %
% --------------------------------------------------------------- %
function saveCircles(all_circles, objFname, objHeaderFlag)

fid = fopen(objFname,'w');
cno = size(all_circles,1);

if (objHeaderFlag == 1)
    fprintf(fid,'%d\n',cno);
end

for i = 1 : cno
    fprintf(fid,'%d\t',all_circles(i,1));
    fprintf(fid,'%d\t',all_circles(i,2));
    fprintf(fid,'%d\t',all_circles(i,3));
    fprintf(fid,'%d\n',all_circles(i,4));
end

fclose(fid);

end

